# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import tarfile

from ..qommon.ctl import Command, make_option


class CmdRestore(Command):
    name = 'restore'

    def __init__(self):
        Command.__init__(
            self, [make_option('--file', metavar='FILENAME', action='store', dest='filename', default=None)]
        )

    def execute(self, base_options, sub_options, args):
        from .. import publisher

        self.config.remove_option('main', 'error_log')
        publisher.WcsPublisher.configure(self.config)
        pub = publisher.WcsPublisher.create_publisher(register_tld_names=False)

        hostname = args[0]
        pub.app_dir = os.path.join(pub.app_dir, hostname)
        if os.path.exists(pub.app_dir):
            print('cannot overwrite site', file=sys.stderr)
            return 1
        os.mkdir(pub.app_dir)

        if not sub_options.filename:
            print('missing --file parameter', file=sys.stderr)
            return 1
        backup_filepath = sub_options.filename

        with tarfile.open(backup_filepath, mode='r:*') as backup:
            for tarinfo in backup:
                if os.path.normpath(tarinfo.name).startswith('..'):
                    continue
                backup.extract(tarinfo, pub.app_dir)


CmdRestore.register()
